@@ -742,11 +742,16 @@ impl fmt::Display for Instruction {
                 name,
                 arguments,
                 data,
-            }) => match data {
-                // FIXME: Handle empty argument lists
-                Some(data) => write!(f, "PRAGMA {} {} {}", name, arguments.join(" "), data),
-                None => write!(f, "PRAGMA {} {}", name, arguments.join(" ")),
-            },
+            }) => {
+                write!(f, "PRAGMA {}", name)?;
+                if !arguments.is_empty() {
+                    write!(f, " {}", arguments.join(" "))?;
+                }
+                if let Some(data) = data {
+                    write!(f, " \"{}\"", data)?;
+                }
+                Ok(())
+            }
             Instruction::RawCapture(RawCapture {
                 blocking,
                 frame,
@@ -782,10 +787,9 @@ impl fmt::Display for Instruction {
             }
             Instruction::WaveformDefinition(WaveformDefinition { name, definition }) => write!(
                 f,
-                "DEFWAVEFORM {}{} {}:\n\t{}",
+                "DEFWAVEFORM {}{}:\n\t{}",
                 name,
                 get_string_parameter_string(&definition.parameters),
-                definition.sample_rate,
                 definition
                     .matrix
                     .iter()
@@ -806,6 +810,42 @@ impl fmt::Display for Instruction {
     }
 }
 
+#[cfg(test)]
+mod test_instruction_display {
+    use super::{Instruction, Pragma};
+
+    #[test]
+    fn pragma() {
+        assert_eq!(
+            Instruction::Pragma(Pragma {
+                name: String::from("INITIAL_REWIRING"),
+                arguments: vec![],
+                data: Some(String::from("PARTIAL")),
+            })
+            .to_string(),
+            "PRAGMA INITIAL_REWIRING \"PARTIAL\""
+        );
+        assert_eq!(
+            Instruction::Pragma(Pragma {
+                name: String::from("LOAD-MEMORY"),
+                arguments: vec![String::from("q0")],
+                data: Some(String::from("addr")),
+            })
+            .to_string(),
+            "PRAGMA LOAD-MEMORY q0 \"addr\""
+        );
+        assert_eq!(
+            Instruction::Pragma(Pragma {
+                name: String::from("PRESERVE_BLOCK"),
+                arguments: vec![],
+                data: None,
+            })
+            .to_string(),
+            "PRAGMA PRESERVE_BLOCK"
+        );
+    }
+}
+
 #[derive(Clone, Debug, Eq, Hash, PartialEq, Serialize, Deserialize)]
 pub enum Qubit {
     Fixed(u64),
@@ -826,5 +866,4 @@ impl fmt::Display for Qubit {
 pub struct Waveform {
     pub matrix: Vec<Expression>,
     pub parameters: Vec<String>,
-    pub sample_rate: f64,
 }
